@@ -27,7 +27,7 @@ _REPOHDLRS = {}
 
 
 class VCSException(Exception):
-    def __init__(self, repoeid, attr, msgid, msgargs):
+    def __init__(self, repoeid, attr, msgid, msgargs=()):
         self.repoeid = repoeid
         self.attr = attr
         self.msgid = msgid
@@ -38,7 +38,9 @@ class VCSException(Exception):
                               self.msgid % self.msgargs)
 
     def to_validation_error(self, translate):
-        msg = translate(self.msgid) % self.msgargs
+        msg = translate(self.msgid)
+        if self.msgargs:
+            msg %= self.msgargs
         return ValidationError(self.repoeid, {self.attr: msg})
 
 
@@ -97,7 +99,7 @@ def import_vcsrepo_content(session, repohdlr, vcsrepo, commitevery=10,
     """synchronize content of given vcs repository"""
     try:
         repohdlr.import_content(vcsrepo, commitevery)
-        # give other threads a chance to run: commit will release the pool, get
+        # give other threads a chance to run: commit will release the cnxset, get
         # it back afterwards
         session.commit()
     except Exception:
@@ -107,14 +109,14 @@ def import_vcsrepo_content(session, repohdlr, vcsrepo, commitevery=10,
         # be rollbacked
         repohdlr.vf_path_cache.clear()
         try:
-            vcsrepo._cw.set_pool() # ensure a pool is set
+            vcsrepo._cw.set_cnxset() # ensure a cnxset is set
             title = vcsrepo.dc_title()
         except Exception:
             title = str(vcsrepo)
         repohdlr.exception(
             'error while importing content for vcs repo %s', title)
         session.rollback()
-    session.set_pool()
+    session.set_cnxset()
 
 
 def set_at_revision(session, reid, parent_lrevs=None, safetybelt=False):
